@@ -1,5 +1,6 @@
 /*
- * Copyright (c) [2012-2014] Novell, Inc.
+ * Copyright (c) [2012-2015] Novell, Inc.
+ * Copyright (c) [2018-2022] SUSE LLC
  *
  * All Rights Reserved.
  *
@@ -34,79 +35,8 @@
 MetaSnappers meta_snappers;
 
 
-RefCounter::RefCounter()
-    : counter(0), last_used(monotonic_time())
-{
-}
-
-
-int
-RefCounter::inc_use_count()
-{
-    boost::lock_guard<boost::mutex> lock(mutex);
-
-    return ++counter;
-}
-
-
-int
-RefCounter::dec_use_count()
-{
-    boost::lock_guard<boost::mutex> lock(mutex);
-
-    assert(counter > 0);
-
-    if (--counter == 0)
-	last_used = monotonic_time();
-
-    return counter;
-}
-
-
-void
-RefCounter::update_use_time()
-{
-    boost::lock_guard<boost::mutex> lock(mutex);
-
-    last_used = monotonic_time();
-}
-
-
-int
-RefCounter::use_count() const
-{
-    boost::lock_guard<boost::mutex> lock(mutex);
-
-    return counter;
-}
-
-
-int
-RefCounter::unused_for() const
-{
-    boost::lock_guard<boost::mutex> lock(mutex);
-
-    if (counter != 0)
-	return 0;
-
-    struct timespec tmp;
-    clock_gettime(CLOCK_MONOTONIC, &tmp);
-
-    return tmp.tv_sec - last_used;
-}
-
-
-time_t
-RefCounter::monotonic_time()
-{
-    struct timespec tmp;
-    clock_gettime(CLOCK_MONOTONIC, &tmp);
-    return tmp.tv_sec;
-}
-
-
 MetaSnapper::MetaSnapper(ConfigInfo& config_info)
-    : config_info(config_info), snapper(NULL)
+    : config_info(config_info)
 {
     set_permissions();
 }
@@ -115,6 +45,7 @@ MetaSnapper::MetaSnapper(ConfigInfo& config_info)
 MetaSnapper::~MetaSnapper()
 {
     delete snapper;
+    snapper = nullptr;
 }
 
 
@@ -134,37 +65,37 @@ MetaSnapper::setConfigInfo(const map<string, string>& raw)
 void
 MetaSnapper::set_permissions()
 {
-    uids.clear();
+    allowed_uids.clear();
 
     vector<string> users;
     if (config_info.getValue(KEY_ALLOW_USERS, users))
     {
-	for (vector<string>::const_iterator it = users.begin(); it != users.end(); ++it)
+	for (const string& user : users)
 	{
 	    uid_t tmp;
-	    if (get_user_uid(it->c_str(), tmp))
-		uids.push_back(tmp);
+	    if (get_user_uid(user.c_str(), tmp))
+		allowed_uids.push_back(tmp);
 	}
     }
 
-    sort(uids.begin(), uids.end());
-    uids.erase(unique(uids.begin(), uids.end()), uids.end());
+    sort(allowed_uids.begin(), allowed_uids.end());
+    allowed_uids.erase(unique(allowed_uids.begin(), allowed_uids.end()), allowed_uids.end());
 
-    gids.clear();
+    allowed_gids.clear();
 
     vector<string> groups;
     if (config_info.getValue(KEY_ALLOW_GROUPS, groups))
     {
-	for (vector<string>::const_iterator it = groups.begin(); it != groups.end(); ++it)
+	for (const string& group : groups)
 	{
 	    gid_t tmp;
-	    if (get_group_gid(it->c_str(), tmp))
-		gids.push_back(tmp);
+	    if (get_group_gid(group.c_str(), tmp))
+		allowed_gids.push_back(tmp);
 	}
     }
 
-    sort(gids.begin(), gids.end());
-    gids.erase(unique(gids.begin(), gids.end()), gids.end());
+    sort(allowed_gids.begin(), allowed_gids.end());
+    allowed_gids.erase(unique(allowed_gids.begin(), allowed_gids.end()), allowed_gids.end());
 }
 
 
@@ -172,7 +103,7 @@ Snapper*
 MetaSnapper::getSnapper()
 {
     if (!snapper)
-	snapper = new Snapper(config_info.getConfigName());
+	snapper = new Snapper(config_info.getConfigName(), "/");
 
     update_use_time();
 
@@ -184,7 +115,7 @@ void
 MetaSnapper::unload()
 {
     delete snapper;
-    snapper = NULL;
+    snapper = nullptr;
 }
 
 
@@ -201,7 +132,7 @@ MetaSnappers::~MetaSnappers()
 void
 MetaSnappers::init()
 {
-    list<ConfigInfo> config_infos = Snapper::getConfigs();
+    list<ConfigInfo> config_infos = Snapper::getConfigs("/");
 
     for (list<ConfigInfo>::iterator it = config_infos.begin(); it != config_infos.end(); ++it)
     {
@@ -210,6 +141,14 @@ MetaSnappers::init()
 }
 
 
+void
+MetaSnappers::unload()
+{
+    for (MetaSnapper& meta_snapper : entries)
+	meta_snapper.unload();
+}
+
+
 MetaSnappers::iterator
 MetaSnappers::find(const string& config_name)
 {
@@ -225,9 +164,9 @@ void
 MetaSnappers::createConfig(const string& config_name, const string& subvolume,
 			   const string& fstype, const string& template_name)
 {
-    Snapper::createConfig(config_name, subvolume, fstype, template_name);
+    Snapper::createConfig(config_name, "/", subvolume, fstype, template_name);
 
-    ConfigInfo config_info = Snapper::getConfig(config_name);
+    ConfigInfo config_info = Snapper::getConfig(config_name, "/");
 
     entries.emplace_back(config_info);
 }
@@ -236,7 +175,7 @@ MetaSnappers::createConfig(const string& config_name, const string& subvolume,
 void
 MetaSnappers::deleteConfig(iterator it)
 {
-    Snapper::deleteConfig(it->configName());
+    Snapper::deleteConfig(it->configName(), "/");
 
     entries.erase(it);
 }
