@@ -1,5 +1,6 @@
 /*
- * Copyright (c) [2012-2013] Novell, Inc.
+ * Copyright (c) [2012-2015] Novell, Inc.
+ * Copyright (c) [2018-2022] SUSE LLC
  *
  * All Rights Reserved.
  *
@@ -28,57 +29,16 @@
 
 #include <snapper/Snapper.h>
 
+#include "RefCounter.h"
+
 
 using namespace std;
 using namespace snapper;
 
 
-class RefCounter : private boost::noncopyable
+struct UnknownConfig : public Exception
 {
-public:
-
-    RefCounter();
-
-    int inc_use_count();
-    int dec_use_count();
-    void update_use_time();
-
-    int use_count() const;
-    int unused_for() const;
-
-private:
-
-    static time_t monotonic_time();
-
-    mutable boost::mutex mutex;
-
-    int counter;
-
-    time_t last_used;
-
-};
-
-
-class RefHolder
-{
-public:
-
-    RefHolder(RefCounter& ref) : ref(ref)
-	{ ref.inc_use_count(); }
-    ~RefHolder()
-	{ ref.dec_use_count(); }
-
-private:
-
-    RefCounter& ref;
-
-};
-
-
-struct UnknownConfig : public std::exception
-{
-    explicit UnknownConfig() throw() {}
-    virtual const char* what() const throw() { return "unknown config"; }
+    explicit UnknownConfig() : Exception("unknown config") {}
 };
 
 
@@ -94,8 +54,8 @@ public:
     const ConfigInfo& getConfigInfo() const { return config_info; }
     void setConfigInfo(const map<string, string>& raw);
 
-    vector<uid_t> uids;
-    vector<gid_t> gids;
+    const vector<uid_t>& get_allowed_uids() const { return allowed_uids; }
+    const vector<gid_t>& get_allowed_gids() const { return allowed_gids; }
 
     Snapper* getSnapper();
 
@@ -109,7 +69,10 @@ private:
 
     ConfigInfo config_info;
 
-    Snapper* snapper;
+    Snapper* snapper = nullptr;
+
+    vector<uid_t> allowed_uids;
+    vector<gid_t> allowed_gids;
 
 };
 
@@ -124,6 +87,8 @@ public:
 
     void init();
 
+    void unload();
+
     typedef list<MetaSnapper>::iterator iterator;
     typedef list<MetaSnapper>::const_iterator const_iterator;
 
