@@ -35,8 +35,16 @@ namespace snapper
     {
     public:
 
+	/**
+	 * Create a comparison.
+	 *
+	 * The mount parameter allows to ensure that the two snapshots are
+	 * mounted for the lifetime of the Comparison object.
+	 */
 	Comparison(const Snapper* snapper, Snapshots::const_iterator snapshot1,
-		   Snapshots::const_iterator snapshot2);
+		   Snapshots::const_iterator snapshot2, bool mount);
+
+	~Comparison();
 
 	const Snapper* getSnapper() const { return snapper; }
 
@@ -46,9 +54,6 @@ namespace snapper
 	Files& getFiles() { return files; }
 	const Files& getFiles() const { return files; }
 
-	void mount() const;
-	void umount() const;
-
 	UndoStatistic getUndoStatistic() const;
         XAUndoStatistic getXAUndoStatistic() const;
 
@@ -61,20 +66,23 @@ namespace snapper
 	void initialize();
 	void create();
 	bool load();
-	void save();
+	void save() const;
 	void filter();
 
+	void do_mount() const;
+	void do_umount() const;
+
 	const Snapper* snapper;
 
-	Snapshots::const_iterator snapshot1;
-	Snapshots::const_iterator snapshot2;
+	const Snapshots::const_iterator snapshot1;
+	const Snapshots::const_iterator snapshot2;
+
+	const bool mount;
 
 	FilePaths file_paths;
 
 	Files files;
 
-	struct AppendHelper;
-
     };
 
 }
