@@ -1,5 +1,6 @@
 /*
- * Copyright (c) [2011-2013] Novell, Inc.
+ * Copyright (c) [2011-2015] Novell, Inc.
+ * Copyright (c) [2016-2018] SUSE LLC
  *
  * All Rights Reserved.
  *
@@ -56,7 +57,7 @@ namespace snapper
     vector<string>
     Filesystem::filter_mount_options(const vector<string>& options)
     {
-	static const char* ign_opt[] = {
+	static const char* ign_opts[] = {
 	    "ro", "rw",
 	    "exec", "noexec", "suid", "nosuid", "dev", "nodev",
 	    "atime", "noatime", "diratime", "nodiratime",
@@ -65,8 +66,8 @@ namespace snapper
 
 	vector<string> ret = options;
 
-	for (size_t i = 0; i < lengthof(ign_opt); ++i)
-	    ret.erase(remove(ret.begin(), ret.end(), ign_opt[i]), ret.end());
+	for (const char* ign_opt : ign_opts)
+	    ret.erase(remove(ret.begin(), ret.end(), ign_opt), ret.end());
 
 	return ret;
     }
@@ -117,7 +118,8 @@ namespace snapper
 	}
 
 	y2err("do not know about fstype '" << fstype << "'");
-	throw InvalidConfigException();
+	SN_THROW(InvalidConfigException());
+	__builtin_unreachable();
     }
 
 
@@ -162,16 +164,52 @@ namespace snapper
 
 
     void
-    Filesystem::createSnapshotOfDefault(unsigned int num, bool read_only) const
+    Filesystem::createSnapshotOfDefault(unsigned int num, bool read_only, bool quota) const
     {
-	throw std::logic_error("not implemented");
+	SN_THROW(UnsupportedException());
+	__builtin_unreachable();
+    }
+
+
+    bool
+    Filesystem::isDefault(unsigned int num) const
+    {
+	return false;
+    }
+
+
+    std::pair<bool, unsigned int>
+    Filesystem::getDefault() const
+    {
+	return std::make_pair(false, 0);
     }
 
 
     void
     Filesystem::setDefault(unsigned int num) const
     {
-	throw std::logic_error("not implemented");
+	SN_THROW(UnsupportedException());
+	__builtin_unreachable();
+    }
+
+
+    std::pair<bool, unsigned int>
+    Filesystem::getActive() const
+    {
+	return std::make_pair(false, 0);
+    }
+
+
+    bool
+    Filesystem::isActive(unsigned int num) const
+    {
+	return false;
+    }
+
+
+    void
+    Filesystem::sync() const
+    {
     }
 
 }
