@@ -1,6 +1,6 @@
 /*
  * Copyright (c) [2011-2015] Novell, Inc.
- * Copyright (c) [2016-2020] SUSE LLC
+ * Copyright (c) [2016-2021] SUSE LLC
  *
  * All Rights Reserved.
  *
@@ -31,6 +31,7 @@
 #include <sys/acl.h>
 #include <acl/libacl.h>
 #include <set>
+#include <regex>
 #include <boost/algorithm/string.hpp>
 
 #include "snapper/Snapper.h"
@@ -45,11 +46,12 @@
 #include "snapper/AsciiFile.h"
 #include "snapper/Exception.h"
 #include "snapper/Hooks.h"
+#ifdef ENABLE_BTRFS
 #include "snapper/Btrfs.h"
 #include "snapper/BtrfsUtils.h"
+#endif
 #ifdef ENABLE_SELINUX
 #include "snapper/Selinux.h"
-#include "snapper/Regex.h"
 #endif
 
 
@@ -156,12 +158,29 @@ namespace snapper
     void
     Snapper::loadIgnorePatterns()
     {
-	const list<string> files = glob(FILTERS_DIR "/*.txt", GLOB_NOSORT);
-	for (list<string>::const_iterator it = files.begin(); it != files.end(); ++it)
+	const vector<string> etc_files = glob(ETC_FILTERS_DIR "/*.txt", GLOB_NOSORT);
+	const vector<string> usr_files = glob(USR_FILTERS_DIR "/*.txt", GLOB_NOSORT);
+
+	// all files from /etc
+	vector<string> files = etc_files;
+
+	// files from /usr where no corresponding file in /etc exists
+	for (const string& usr_file : usr_files)
+	{
+	    string b1 = basename(usr_file);
+
+	    if (none_of(etc_files.begin(), etc_files.end(), [&b1](const string& etc_file) {
+		string b2 = basename(etc_file);
+		return b1 == b2;
+	    }))
+		files.push_back(usr_file);
+	}
+
+	for (const string& file : files)
 	{
 	    try
 	    {
-		AsciiFileReader asciifile(*it);
+		AsciiFileReader asciifile(file);
 
 		string line;
 		while (asciifile.getline(line))
@@ -333,7 +352,13 @@ namespace snapper
 	    }
 	}
 
-	if (access(string(CONFIG_TEMPLATE_DIR "/" + template_name).c_str(), R_OK) != 0)
+	string template_file;
+
+	try
+	{
+	    template_file = locate_file(template_name, ETC_CONFIG_TEMPLATE_DIR, USR_CONFIG_TEMPLATE_DIR);
+	}
+	catch (...)
 	{
 	    SN_THROW(CreateConfigFailedException("cannot access template config"));
 	}
@@ -372,7 +397,7 @@ namespace snapper
 
 	try
 	{
-	    SysconfigFile config(CONFIG_TEMPLATE_DIR "/" + template_name);
+	    SysconfigFile config(template_file);
 
 	    config.setName(CONFIGS_DIR "/" + config_name);
 
@@ -820,13 +845,22 @@ namespace snapper
     FreeSpaceData
     Snapper::queryFreeSpaceData() const
     {
+#ifdef ENABLE_BTRFS
+
 	const Btrfs* btrfs = dynamic_cast<const Btrfs*>(getFilesystem());
 	if (!btrfs)
 	    SN_THROW(FreeSpaceException("free space only supported with btrfs"));
 
 	SDir general_dir = btrfs->openGeneralDir();
 
-	filesystem->sync();
+	try
+	{
+	    filesystem->sync();
+	}
+	catch (...)
+	{
+	    SN_THROW(FreeSpaceException("filesystem sync failed"));
+	}
 
 	FreeSpaceData free_space_data;
 	std::tie(free_space_data.size, free_space_data.free) = general_dir.statvfs();
@@ -837,6 +871,13 @@ namespace snapper
 	    SN_THROW(FreeSpaceException("impossible free space values"));
 
 	return free_space_data;
+
+#else
+
+        SN_THROW(QuotaException("not implemented"));
+        __builtin_unreachable();
+
+#endif
     }
 
 
@@ -874,8 +915,8 @@ namespace snapper
     Snapper::syncSelinuxContextsInInfosDir(bool skip_snapshot_dir) const
     {
 #ifdef ENABLE_SELINUX
-	Regex rx("^[0-9]+$");
-	Regex rx_filelist("^filelist-[0-9]+.txt$");
+	static const regex rx("[0-9]+", regex::extended);
+	static const regex rx_filelist("filelist-[0-9]+.txt", regex::extended);
 
 	y2deb("Syncing Selinux contexts in infos dir");
 
@@ -884,7 +925,7 @@ namespace snapper
 	vector<string> infos = infos_dir.entries();
 	for (vector<string>::const_iterator it1 = infos.begin(); it1 != infos.end(); ++it1)
 	{
-	    if (!rx.match(*it1))
+	    if (!regex_match(*it1, rx))
 		continue;
 
 	    SDir info_dir(infos_dir, *it1);
@@ -902,7 +943,7 @@ namespace snapper
 	    vector<string> info_content = info_dir.entries();
 	    for (vector<string>::const_iterator it2 = info_content.begin(); it2 != info_content.end(); ++it2)
 	    {
-		if (!rx_filelist.match(*it2))
+		if (!regex_match(*it2, rx_filelist))
 		    continue;
 
 		SFile fl(info_dir, *it2);
